using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;
using GPUTools.Cloth.Scripts;


namespace Stopper {
	public class ClothColliders: MVRScript {
		DAZCharacterSelector characterSelector;
		
		DAZMergedSkinV2 skin;

		GameObject clothMeshContainer;
	
		List<string> allowedRigidBodies = new List<string>(){
			"head", "neck",
			"abdomen", "chest", "hip",
			"collar", "shldr", "forearm", "hand",
			"thigh", "shin", "foot"
		};
		
		string selectedClothingUid;
		string selectedRigidbody;
		
		JSONStorableBool showCollider;
		
		Dictionary<string, Rigidbody> rigidbodyMap = new Dictionary<string, Rigidbody>();
		
		List<UnityEngine.Object> createdObjects = new List<UnityEngine.Object>();
		
		MeshCollider meshCollider;
		MeshRenderer meshRenderer;
		
		public override void Init() {
			var geometry = containingAtom.gameObject.transform.Find("rescale2/PhysicsModel");
			
			if(geometry == null)
				return;
			
			foreach(Rigidbody rb in geometry.gameObject.GetComponentsInChildren<Rigidbody>()){
				if(rb.name.ToLower().Contains("collider"))
					continue;
				
				foreach(string allowedRB in allowedRigidBodies){
					if(rb.name.ToLower().Contains(allowedRB)){
						rigidbodyMap[rb.name] = rb;
					}
				}
			}
			
			clothMeshContainer = CreateChildObject(containingAtom.gameObject, "ClothMeshCollider");
			
			selectedClothingUid = "Clothing Creator";
			
			characterSelector = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
			
			JSONStorableStringChooser clothingChooser = new JSONStorableStringChooser(
					"Target Clothing", 
					containingAtom.GetComponentsInChildren<DAZClothingItem>().Select(c => c.uid).ToList(),
					selectedClothingUid, 
					"Target Clothing", 
					(string uid) => {
						selectedClothingUid = uid;
					});
			CreateScrollablePopup(clothingChooser, false);
			
			JSONStorableStringChooser rigidbodyChooser = new JSONStorableStringChooser(
					"Parent Bodypart", 
					rigidbodyMap.Keys.ToList(),
					null, 
					"Parent Bodypart", 
					(string rb) => {
						selectedRigidbody = rb;
					});
			CreateScrollablePopup(rigidbodyChooser, false);
			
			var addColliderButton = CreateButton("Add Mesh Collider", false);
			addColliderButton.button.onClick.AddListener(() => {
				AddClothingMesh(selectedClothingUid, selectedRigidbody);
			});
			
			showCollider = new JSONStorableBool("Show Collider", false);
			
			var showColliderToggle = CreateToggle(showCollider, false);
			showColliderToggle.toggle.onValueChanged.AddListener((val) => {
				if(meshRenderer != null){
					meshRenderer.enabled = val;
				}
			});
		}
		
		
		private void AddClothingMesh(string clothingUid, string rigidbodyName){
			if(clothingUid == null || rigidbodyName == null){
				return;
			}
			
			var clothingItem = characterSelector.GetClothingItem(selectedClothingUid);
			
			if(clothingItem == null){
				SuperController.LogMessage($"Could not find Clothing {selectedClothingUid}");
				return;
			}
			
			var skinWrap = clothingItem.gameObject.GetComponentInChildren<DAZSkinWrap>();
			
			var rb = rigidbodyMap[rigidbodyName];
			if(clothingItem == null){
				SuperController.LogMessage($"Could not find Rigidbody {rigidbodyName}");
				return;
			}
			var rendererContainer = CreateChildObject(rb.gameObject, $"{clothingUid}");
			
			if(meshCollider != null){
				OnDestroy();
			}
			
			var bone = rb.gameObject.GetComponent<DAZBone>();
			
			Mesh mesh = new Mesh();
		
			var vertices = (Vector3[])skinWrap.Mesh.vertices.Clone();
			
			for(int i = 0; i < vertices.Length; i++){
				vertices[i] = vertices[i] - bone.importWorldPosition;
				
				//var rotation = Quaternion.Inverse(Quaternion.LookRotation(bone.importWorldOrientation, rb.transform.up));
				vertices[i] = Quaternion.Inverse(bone.startingRotationRelativeToRoot)*vertices[i];
			}
			mesh.vertices = vertices;
			mesh.uv = skinWrap.Mesh.uv;
			mesh.triangles = skinWrap.Mesh.triangles;
			
			meshCollider = rendererContainer.AddComponent<MeshCollider>();
			meshCollider.gameObject.layer = 29;
			meshCollider.sharedMesh = mesh;
			meshCollider.convex = true;
			meshCollider.enabled = true;
			
			createdObjects.Add(meshCollider);
			
			
			foreach(Collider collider in rb.gameObject.GetComponentsInChildren<Collider>()){
				Physics.IgnoreCollision(meshCollider, collider, true);
			}
			
			rendererContainer.transform.position = rb.transform.position;
			rendererContainer.transform.rotation = rb.transform.rotation;
			
			meshRenderer = rendererContainer.gameObject.AddComponent<MeshRenderer>();
			var meshFilter = rendererContainer.gameObject.AddComponent<MeshFilter>();
			meshFilter.mesh = mesh;
			meshRenderer.enabled = showCollider.val;
			
			Material lineMaterial = new Material(Shader.Find("Battlehub/RTGizmos/Handles"));
			lineMaterial.SetFloat("_Offset", 1f);
			lineMaterial.SetFloat("_MinAlpha", 0f);
			
			Color test = Color.green;
			
			test.a = 0.5f;
			lineMaterial.color = test;
			
			
			meshRenderer.material = lineMaterial;
			
			SuperController.LogMessage($"{skinWrap} {rb}");
		}
		
		private void Start(){
			
		}
		
		private void Update(){
		/*	RaycastHit hit;
			Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
			if (meshCollider.Raycast(ray, out hit, 100f)) {
				
		//		SuperController.LogMessage($"{hit}");
			} else {
	//			SuperController.LogMessage($"miss");
			}*/
		}
		
		
			
		private void OnDisable()
		{
			if(meshCollider != null){
				meshCollider.enabled = false;
			}
		}
		
		private void OnDestroy(){
			foreach(var o in createdObjects){
				UnityEngine.Object.Destroy(o);
			}
		}
		
		private JSONStorableFloat AddSlider(string name, float defaultVal, float min, float max, JSONStorableFloat.SetFloatCallback onChange = null, bool rangeAdjust = false, bool rightSide = false){
			JSONStorableFloat storableFloat = new JSONStorableFloat(name, defaultVal, min, max, true, true);
			storableFloat.storeType = JSONStorableParam.StoreType.Full;
			UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
			uiSlider.rangeAdjustEnabled = rangeAdjust;
			
			if(onChange != null){
				storableFloat.setCallbackFunction = onChange;
			}
			
			return storableFloat;
		}
		
		private GameObject CreateChildObject(GameObject parent, string name) {
			GameObject gameObject = new GameObject(name);
			gameObject.transform.parent = parent.transform;
			
			createdObjects.Add(gameObject);
		
			return gameObject;
		}
	}
}
